@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_libpq_ifce.c,v $ $Revision: 4702 $ $Date: 2011-07-29 13:31:47 +0200 (Fri, 29 Jul 2011) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_libpq_ifce.c,v $ $Revision: 8692 $ $Date: 2013-04-22 11:33:34 +0200 (Mon, 22 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -1344,8 +1344,7 @@ DBLISTPTR *dblistptr;
 		 STATUS, ERRSTRING, GROUPS \
 		FROM dpm_req \
 		WHERE ctime = %d \
-		AND (status = %d OR status >= %d) \
-		FOR UPDATE";
+		AND (status = %d OR status >= %d)";
 	static char query4updlim1[] =
 		"SELECT ROWID, \
 		 R_ORDINAL, R_TOKEN, R_UID, \
@@ -1357,8 +1356,7 @@ DBLISTPTR *dblistptr;
 		FROM dpm_req \
 		WHERE ctime <= %d \
 		AND (status = %d OR status >= %d) \
-		LIMIT %d \
-		FOR UPDATE";
+		LIMIT %d";
 	static char query4updlim2[] =
 		"SELECT ROWID, \
 		 R_ORDINAL, R_TOKEN, R_UID, \
@@ -1370,8 +1368,7 @@ DBLISTPTR *dblistptr;
 		FROM dpm_req \
 		WHERE ctime >= %d AND ctime <= %d \
 		AND (status = %d OR status >= %d) \
-		LIMIT %d \
-		FOR UPDATE";
+		LIMIT %d";
 	PGresult *res;
 	char sql_stmt[1024];
 
