@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_mysql_ifce.c,v $ $Revision: 7960 $ $Date: 2012-12-04 09:52:23 +0100 (Tue, 04 Dec 2012) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_mysql_ifce.c,v $ $Revision: 8698 $ $Date: 2013-04-23 09:46:04 +0200 (Tue, 23 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -46,6 +46,20 @@ unsigned long max;
 	return mysql_real_escape_string (mysql, to, from, num);
 }
 
+int dm_mysql_query(MYSQL *mysql, const char *stmt_str) {
+	static char func[] = "dm_mysql_query";
+	int ret;
+	while(1) {
+		ret = mysql_query (mysql, stmt_str);
+		if (ret && mysql_errno (mysql) == ER_LOCK_WAIT_TIMEOUT) {
+			dpmlogit (func, "Restarting query after lock wait timeout\n");
+			continue;
+		}
+		break;
+	}
+	return ret;
+}
+
 static int
 dm_snprintf(char *str, size_t size, const char *format, ...)
 {  
@@ -150,7 +164,7 @@ struct dpm_dbfd *dbfd;
 dpm_abort_tr(dbfd)
 struct dpm_dbfd *dbfd;
 {
-	(void) mysql_query (&dbfd->mysql, "ROLLBACK");
+	(void) dm_mysql_query (&dbfd->mysql, "ROLLBACK");
 	dbfd->tr_started = 0;
 	return (0);
 }
@@ -169,22 +183,40 @@ dpm_dbrec_addr *rec_addr;
 struct dpm_copy_filereq *cpr_entry;
 {
 	int i = 0;
+	char *p;
+	memset (cpr_entry, 0, sizeof(struct dpm_copy_filereq));
 
-	if (lock)
-		strcpy (*rec_addr, row[i++]);
-	strcpy (cpr_entry->r_token, row[i++]);
-	cpr_entry->f_ordinal = atoi (row[i++]);
-	strcpy (cpr_entry->from_surl, row[i++]);
-	strcpy (cpr_entry->to_surl, row[i++]);
-	cpr_entry->f_lifetime = atoi (row[i++]);
-	cpr_entry->f_type = *row[i++];
-	strcpy (cpr_entry->s_token, row[i++]);
-	cpr_entry->ret_policy = *row[i++];
-	cpr_entry->ac_latency = *row[i++];
-	cpr_entry->flags = atoi (row[i++]);
-	cpr_entry->actual_size = strtou64 (row[i++]);
-	cpr_entry->status = atoi (row[i++]);
-	strcpy (cpr_entry->errstring, row[i]);
+	if (lock) {
+		memset (rec_addr, 0, sizeof(dpm_dbrec_addr));
+		if ((p=row[i++]))
+			strncat (*rec_addr, p, sizeof(dpm_dbrec_addr)-1);
+	}
+	if ((p=row[i++]))
+		strncat (cpr_entry->r_token, p, sizeof(cpr_entry->r_token)-1);
+	if ((p=row[i++]))
+		cpr_entry->f_ordinal = atoi (p);
+	if ((p=row[i++]))
+		strncat (cpr_entry->from_surl, p, sizeof(cpr_entry->from_surl)-1);
+	if ((p=row[i++]))
+		strncat (cpr_entry->to_surl, p, sizeof(cpr_entry->to_surl)-1);
+	if ((p=row[i++]))
+		cpr_entry->f_lifetime = atoi (p);
+	if ((p=row[i++]))
+		cpr_entry->f_type = *p;
+	if ((p=row[i++]))
+		strncat (cpr_entry->s_token, p, sizeof(cpr_entry->s_token)-1);
+	if ((p=row[i++]))
+		cpr_entry->ret_policy = *p;
+	if ((p=row[i++]))
+		cpr_entry->ac_latency = *p;
+	if ((p=row[i++]))
+		cpr_entry->flags = atoi (p);
+	if ((p=row[i++]))
+		cpr_entry->actual_size = strtou64 (p);
+	if ((p=row[i++]))
+		cpr_entry->status = atoi (p);
+	if ((p=row[i]))
+		strncat (cpr_entry->errstring, p, sizeof(cpr_entry->errstring)-1);
 }
 
 dpm_decode_fs_entry(row, lock, rec_addr, fs_entry)
@@ -194,14 +226,24 @@ dpm_dbrec_addr *rec_addr;
 struct dpm_fs *fs_entry;
 {
 	int i = 0;
+	char *p;
+	memset (fs_entry, 0, sizeof(struct dpm_fs));
 
-	if (lock)
-		strcpy (*rec_addr, row[i++]);
-	strcpy (fs_entry->poolname, row[i++]);
-	strcpy (fs_entry->server, row[i++]);
-	strcpy (fs_entry->fs, row[i++]);
-	fs_entry->status = atoi (row[i++]);
-	fs_entry->weight = atoi (row[i]);
+	if (lock) {
+		memset (rec_addr, 0, sizeof(dpm_dbrec_addr));
+		if ((p=row[i++]))
+			strncat (*rec_addr, p, sizeof(dpm_dbrec_addr)-1);
+	}
+	if ((p=row[i++]))
+		strncat (fs_entry->poolname, p, sizeof(fs_entry->poolname)-1);
+	if ((p=row[i++]))
+		strncat (fs_entry->server, p, sizeof(fs_entry->server)-1);
+	if ((p=row[i++]))
+		strncat (fs_entry->fs, p, sizeof(fs_entry->fs)-1);
+	if ((p=row[i++]))
+		fs_entry->status = atoi (p);
+	if ((p=row[i]))
+		fs_entry->weight = atoi (p);
 }
 
 dpm_decode_gfr_entry(row, lock, rec_addr, gfr_entry)
@@ -211,24 +253,44 @@ dpm_dbrec_addr *rec_addr;
 struct dpm_get_filereq *gfr_entry;
 {
 	int i = 0;
+	char *p;
+	memset (gfr_entry, 0, sizeof(struct dpm_get_filereq));
 
-	if (lock)
-		strcpy (*rec_addr, row[i++]);
-	strcpy (gfr_entry->r_token, row[i++]);
-	gfr_entry->f_ordinal = atoi (row[i++]);
-	gfr_entry->r_uid = atoi (row[i++]);
-	strcpy (gfr_entry->from_surl, row[i++]);
-	strcpy (gfr_entry->protocol, row[i++]);
-	gfr_entry->lifetime = atoi (row[i++]);
-	gfr_entry->f_type = *row[i++];
-	strcpy (gfr_entry->s_token, row[i++]);
-	gfr_entry->ret_policy = *row[i++];
-	gfr_entry->flags = atoi (row[i++]);
-	strcpy (gfr_entry->server, row[i++]);
-	strcpy (gfr_entry->pfn, row[i++]);
-	gfr_entry->actual_size = strtou64 (row[i++]);
-	gfr_entry->status = atoi (row[i++]);
-	strcpy (gfr_entry->errstring, row[i]);
+	if (lock) {
+		memset (rec_addr, 0, sizeof(dpm_dbrec_addr));
+		if ((p=row[i++]))
+			strncat (*rec_addr, p, sizeof(dpm_dbrec_addr)-1);
+	}
+	if ((p=row[i++]))
+		strncat (gfr_entry->r_token, p, sizeof(gfr_entry->r_token)-1);
+	if ((p=row[i++]))
+		gfr_entry->f_ordinal = atoi (p);
+	if ((p=row[i++]))
+		gfr_entry->r_uid = atoi (p);
+	if ((p=row[i++]))
+		strncat (gfr_entry->from_surl, p, sizeof(gfr_entry->from_surl)-1);
+	if ((p=row[i++]))
+		strncat (gfr_entry->protocol, p, sizeof(gfr_entry->protocol)-1);
+	if ((p=row[i++]))
+		gfr_entry->lifetime = atoi (p);
+	if ((p=row[i++]))
+		gfr_entry->f_type = *p;
+	if ((p=row[i++]))
+		strncat (gfr_entry->s_token, p, sizeof(gfr_entry->s_token)-1);
+	if ((p=row[i++]))
+		gfr_entry->ret_policy = *p;
+	if ((p=row[i++]))
+		gfr_entry->flags = atoi (p);
+	if ((p=row[i++]))
+		strncat (gfr_entry->server, p, sizeof(gfr_entry->server)-1);
+	if ((p=row[i++]))
+		strncat (gfr_entry->pfn, p, sizeof(gfr_entry->pfn)-1);
+	if ((p=row[i++]))
+		gfr_entry->actual_size = strtou64 (p);
+	if ((p=row[i++]))
+		gfr_entry->status = atoi (p);
+	if ((p=row[i]))
+		strncat (gfr_entry->errstring, p, sizeof(gfr_entry->errstring)-1);
 }
 
 dpm_decode_pfr_entry(row, lock, rec_addr, pfr_entry)
@@ -238,25 +300,46 @@ dpm_dbrec_addr *rec_addr;
 struct dpm_put_filereq *pfr_entry;
 {
 	int i = 0;
+	char *p;
+	memset (pfr_entry, 0, sizeof(struct dpm_put_filereq));
 
-	if (lock)
-		strcpy (*rec_addr, row[i++]);
-	strcpy (pfr_entry->r_token, row[i++]);
-	pfr_entry->f_ordinal = atoi (row[i++]);
-	strcpy (pfr_entry->to_surl, row[i++]);
-	strcpy (pfr_entry->protocol, row[i++]);
-	pfr_entry->lifetime = atoi (row[i++]);
-	pfr_entry->f_lifetime = atoi (row[i++]);
-	pfr_entry->f_type = *row[i++];
-	strcpy (pfr_entry->s_token, row[i++]);
-	pfr_entry->ret_policy = *row[i++];
-	pfr_entry->ac_latency = *row[i++];
-	pfr_entry->requested_size = strtou64 (row[i++]);
-	strcpy (pfr_entry->server, row[i++]);
-	strcpy (pfr_entry->pfn, row[i++]);
-	pfr_entry->actual_size = strtou64 (row[i++]);
-	pfr_entry->status = atoi (row[i++]);
-	strcpy (pfr_entry->errstring, row[i]);
+	if (lock) {
+		memset (rec_addr, 0, sizeof(dpm_dbrec_addr));
+		if ((p=row[i++]))
+			strncat (*rec_addr, p, sizeof(dpm_dbrec_addr)-1);
+	}
+	if ((p=row[i++]))
+		strncat (pfr_entry->r_token, p, sizeof(pfr_entry->r_token)-1);
+	if ((p=row[i++]))
+		pfr_entry->f_ordinal = atoi (p);
+	if ((p=row[i++]))
+		strncat (pfr_entry->to_surl, p, sizeof(pfr_entry->to_surl)-1);
+	if ((p=row[i++]))
+		strncat (pfr_entry->protocol, p, sizeof(pfr_entry->protocol)-1);
+	if ((p=row[i++]))
+		pfr_entry->lifetime = atoi (p);
+	if ((p=row[i++]))
+		pfr_entry->f_lifetime = atoi (p);
+	if ((p=row[i++]))
+		pfr_entry->f_type = *p;
+	if ((p=row[i++]))
+		strncat (pfr_entry->s_token, p, sizeof(pfr_entry->s_token)-1);
+	if ((p=row[i++]))
+		pfr_entry->ret_policy = *p;
+	if ((p=row[i++]))
+		pfr_entry->ac_latency = *p;
+	if ((p=row[i++]))
+		pfr_entry->requested_size = strtou64 (p);
+	if ((p=row[i++]))
+		strncat (pfr_entry->server, p, sizeof(pfr_entry->server)-1);
+	if ((p=row[i++]))
+		strncat (pfr_entry->pfn, p, sizeof(pfr_entry->pfn)-1);
+	if ((p=row[i++]))
+		pfr_entry->actual_size = strtou64 (p);
+	if ((p=row[i++]))
+		pfr_entry->status = atoi (p);
+	if ((p=row[i]))
+		strncat (pfr_entry->errstring, p, sizeof(pfr_entry->errstring)-1);
 }
 
 dpm_decode_pool_entry(row, lock, rec_addr, pool_entry)
@@ -266,24 +349,45 @@ dpm_dbrec_addr *rec_addr;
 struct dpm_pool *pool_entry;
 {
 	int i = 0;
+	char *p;
+	memset (pool_entry, 0, sizeof(struct dpm_pool));
 
-	if (lock)
-		strcpy (*rec_addr, row[i++]);
-	strcpy (pool_entry->poolname, row[i++]);
-	pool_entry->defsize = strtou64 (row[i++]);
-	pool_entry->gc_start_thresh = atoi (row[i++]);
-	pool_entry->gc_stop_thresh = atoi (row[i++]);
-	pool_entry->def_lifetime = atoi (row[i++]);
-	pool_entry->defpintime = atoi (row[i++]);
-	pool_entry->max_lifetime = atoi (row[i++]);
-	pool_entry->maxpintime = atoi (row[i++]);
-	strcpy (pool_entry->fss_policy, row[i++]);
-	strcpy (pool_entry->gc_policy, row[i++]);
-	strcpy (pool_entry->mig_policy, row[i++]);
-	strcpy (pool_entry->rs_policy, row[i++]);
-	pool_entry->gids = Cdecode_groups (row[i++], &pool_entry->nbgids);
-	pool_entry->ret_policy = *row[i++];
-	pool_entry->s_type = *row[i];
+	if (lock) {
+		memset (rec_addr, 0, sizeof(dpm_dbrec_addr));
+		if ((p=row[i++]))
+			strncat (*rec_addr, p, sizeof(dpm_dbrec_addr)-1);
+	}
+	if ((p=row[i++]))
+		strncat (pool_entry->poolname, p, sizeof(pool_entry->poolname)-1);
+	if ((p=row[i++]))
+		pool_entry->defsize = strtou64 (p);
+	if ((p=row[i++]))
+		pool_entry->gc_start_thresh = atoi (p);
+	if ((p=row[i++]))
+		pool_entry->gc_stop_thresh = atoi (p);
+	if ((p=row[i++]))
+		pool_entry->def_lifetime = atoi (p);
+	if ((p=row[i++]))
+		pool_entry->defpintime = atoi (p);
+	if ((p=row[i++]))
+		pool_entry->max_lifetime = atoi (p);
+	if ((p=row[i++]))
+		pool_entry->maxpintime = atoi (p);
+	if ((p=row[i++]))
+		strncat (pool_entry->fss_policy, p, sizeof(pool_entry->fss_policy)-1);
+	if ((p=row[i++]))
+		strncat (pool_entry->gc_policy, p, sizeof(pool_entry->gc_policy)-1);
+	if ((p=row[i++]))
+		strncat (pool_entry->mig_policy, p, sizeof(pool_entry->mig_policy)-1);
+	if ((p=row[i++]))
+		strncat (pool_entry->rs_policy, p, sizeof(pool_entry->rs_policy)-1);
+	pool_entry->gids = Cdecode_groups (row[i] ? row[i] : "",
+	    &pool_entry->nbgids);
+	i++;
+	if ((p=row[i++]))
+		pool_entry->ret_policy = *p;
+	if ((p=row[i]))
+		pool_entry->s_type = *p;
 }
 
 dpm_decode_spcmd_entry(row, lock, rec_addr, dpm_spcmd)
@@ -293,25 +397,44 @@ dpm_dbrec_addr *rec_addr;
 struct dpm_space_reserv *dpm_spcmd;
 {
 	int i = 0;
+	char *p;
+	memset (dpm_spcmd, 0, sizeof(struct dpm_space_reserv));
 
-	if (lock)
-		strcpy (*rec_addr, row[i++]);
-	strcpy (dpm_spcmd->s_token, row[i++]);
-	strcpy (dpm_spcmd->client_dn, row[i++]);
-	dpm_spcmd->s_uid = atoi (row[i++]);
-	dpm_spcmd->s_gid = atoi (row[i++]);
-	dpm_spcmd->ret_policy = *row[i++];
-	dpm_spcmd->ac_latency = *row[i++];
-	dpm_spcmd->s_type = *row[i++];
-	strcpy (dpm_spcmd->u_token, row[i++]);
-	dpm_spcmd->t_space = strtou64 (row[i++]);
-	dpm_spcmd->g_space = strtou64 (row[i++]);
-	dpm_spcmd->u_space = strtoi64 (row[i++]);
-	strcpy (dpm_spcmd->poolname, row[i] ? row[i] : "");
-	i++;
-	dpm_spcmd->assign_time = atoi (row[i++]);
-	dpm_spcmd->expire_time = atoi (row[i++]);
-	strcpy (dpm_spcmd->groups, row[i]);
+	if (lock) {
+		memset (rec_addr, 0, sizeof(dpm_dbrec_addr));
+		if ((p=row[i++]))
+			strncat (*rec_addr, p, sizeof(dpm_dbrec_addr)-1);
+	}
+	if ((p=row[i++]))
+		strncat (dpm_spcmd->s_token, p, sizeof(dpm_spcmd->s_token)-1);
+	if ((p=row[i++]))
+		strncat (dpm_spcmd->client_dn, p, sizeof(dpm_spcmd->client_dn)-1);
+	if ((p=row[i++]))
+		dpm_spcmd->s_uid = atoi (p);
+	if ((p=row[i++]))
+		dpm_spcmd->s_gid = atoi (p);
+	if ((p=row[i++]))
+		dpm_spcmd->ret_policy = *p;
+	if ((p=row[i++]))
+		dpm_spcmd->ac_latency = *p;
+	if ((p=row[i++]))
+		dpm_spcmd->s_type = *p;
+	if ((p=row[i++]))
+		strncat (dpm_spcmd->u_token, p, sizeof(dpm_spcmd->u_token)-1);
+	if ((p=row[i++]))
+		dpm_spcmd->t_space = strtou64 (p);
+	if ((p=row[i++]))
+		dpm_spcmd->g_space = strtou64 (p);
+	if ((p=row[i++]))
+		dpm_spcmd->u_space = strtoi64 (p);
+	if ((p=row[i++]))
+		strncat (dpm_spcmd->poolname, p, sizeof(dpm_spcmd->poolname)-1);
+	if ((p=row[i++]))
+		dpm_spcmd->assign_time = atoi (p);
+	if ((p=row[i++]))
+		dpm_spcmd->expire_time = atoi (p);
+	if ((p=row[i]))
+		strncat (dpm_spcmd->groups, p, sizeof(dpm_spcmd->groups)-1);
 }
 
 dpm_decode_xferreq_entry(row, lock, rec_addr, dpm_req)
@@ -321,26 +444,48 @@ dpm_dbrec_addr *rec_addr;
 struct dpm_req *dpm_req;
 {
 	int i = 0;
+	char *p;
+	memset (dpm_req, 0, sizeof(struct dpm_req));
 
-	if (lock)
-		strcpy (*rec_addr, row[i++]);
-	dpm_req->r_ordinal = atoi (row[i++]);
-	strcpy (dpm_req->r_token, row[i++]);
-	dpm_req->r_uid = atoi (row[i++]);
-	dpm_req->r_gid = atoi (row[i++]);
-	strcpy (dpm_req->client_dn, row[i++]);
-	strcpy (dpm_req->clienthost, row[i++]);
-	dpm_req->r_type = *row[i++];
-	strcpy (dpm_req->u_token, row[i++]);
-	dpm_req->flags = atoi (row[i++]);
-	dpm_req->retrytime = atoi (row[i++]);
-	dpm_req->nbreqfiles = atoi (row[i++]);
-	dpm_req->ctime = atoi (row[i++]);
-	dpm_req->stime = atoi (row[i++]);
-	dpm_req->etime = atoi (row[i++]);
-	dpm_req->status = atoi (row[i++]);
-	strcpy (dpm_req->errstring, row[i++]);
-	strcpy (dpm_req->groups, row[i]);
+	if (lock) {
+		memset (rec_addr, 0, sizeof(dpm_dbrec_addr));
+		if ((p=row[i++]))
+			strncat (*rec_addr, p, sizeof(dpm_dbrec_addr)-1);
+	}
+	if ((p=row[i++]))
+		dpm_req->r_ordinal = atoi (p);
+	if ((p=row[i++]))
+		strncat (dpm_req->r_token, p, sizeof(dpm_req->r_token)-1);
+	if ((p=row[i++]))
+		dpm_req->r_uid = atoi (p);
+	if ((p=row[i++]))
+		dpm_req->r_gid = atoi (p);
+	if ((p=row[i++]))
+		strncat (dpm_req->client_dn, p, sizeof(dpm_req->client_dn)-1);
+	if ((p=row[i++]))
+		strncat (dpm_req->clienthost, p, sizeof(dpm_req->clienthost)-1);
+	if ((p=row[i++]))
+		dpm_req->r_type = *p;
+	if ((p=row[i++]))
+		strncat (dpm_req->u_token, p, sizeof(dpm_req->u_token)-1);
+	if ((p=row[i++]))
+		dpm_req->flags = atoi (p);
+	if ((p=row[i++]))
+		dpm_req->retrytime = atoi (p);
+	if ((p=row[i++]))
+		dpm_req->nbreqfiles = atoi (p);
+	if ((p=row[i++]))
+		dpm_req->ctime = atoi (p);
+	if ((p=row[i++]))
+		dpm_req->stime = atoi (p);
+	if ((p=row[i++]))
+		dpm_req->etime = atoi (p);
+	if ((p=row[i++]))
+		dpm_req->status = atoi (p);
+	if ((p=row[i++]))
+		strncat (dpm_req->errstring, p, sizeof(dpm_req->errstring)-1);
+	if ((p=row[i]))
+		strncat (dpm_req->groups, p, sizeof(dpm_req->groups)-1);
 }
 
 dpm_delete_cprs_by_token(dbfd, r_token)
@@ -357,7 +502,7 @@ char *r_token;
 	dm_escape_string (&dbfd->mysql, escaped_token, r_token,
 	    strlen (r_token), sizeof(escaped_token));
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, escaped_token);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -375,7 +520,7 @@ dpm_dbrec_addr *rec_addr;
 
 	strcpy (func, "dpm_delete_fs_entry");
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, *rec_addr);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -396,7 +541,7 @@ char *r_token;
 	dm_escape_string (&dbfd->mysql, escaped_token, r_token,
 	    strlen (r_token), sizeof(escaped_token));
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, escaped_token);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -414,7 +559,7 @@ dpm_dbrec_addr *rec_addr;
 
 	strcpy (func, "dpm_delete_pending_entry");
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, *rec_addr);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -435,7 +580,7 @@ char *r_token;
 	dm_escape_string (&dbfd->mysql, escaped_token, r_token,
 	    strlen (r_token), sizeof(escaped_token));
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, escaped_token);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -453,7 +598,7 @@ dpm_dbrec_addr *rec_addr;
 
 	strcpy (func, "dpm_delete_pool_entry");
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, *rec_addr);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -471,7 +616,7 @@ dpm_dbrec_addr *rec_addr;
 
 	strcpy (func, "dpm_delete_req_entry");
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, *rec_addr);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -489,7 +634,7 @@ dpm_dbrec_addr *rec_addr;
 
 	strcpy (func, "dpm_delete_spcmd_entry");
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), delete_stmt, *rec_addr);
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "DELETE", dbfd);
 		return (-1);
 	}
@@ -499,7 +644,7 @@ dpm_dbrec_addr *rec_addr;
 dpm_end_tr(dbfd)
 struct dpm_dbfd *dbfd;
 {
-	(void) mysql_query (&dbfd->mysql, "COMMIT");
+	(void) dm_mysql_query (&dbfd->mysql, "COMMIT");
 	dbfd->tr_started = 0;
 	return (0);
 }
@@ -510,14 +655,21 @@ struct dpm_dbfd *dbfd;
 char *sql_stmt;
 MYSQL_RES **res;
 {
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
-		*res = NULL;
-		dpm_mysql_error (func, "mysql_query", dbfd);
-		return (-1);
-	}
-	if ((*res = mysql_store_result (&dbfd->mysql)) == NULL) {
-		dpm_mysql_error (func, "mysql_store_res", dbfd);
-		return (-1);
+	while(1) {
+		if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
+			*res = NULL;
+			dpm_mysql_error (func, "mysql_query", dbfd);
+			return (-1);
+		}
+		if ((*res = mysql_store_result (&dbfd->mysql)) == NULL) {
+			if (mysql_errno (&dbfd->mysql) == ER_LOCK_WAIT_TIMEOUT) {
+				dpmlogit (func, "Restarting query after lock wait timeout\n");
+				continue;
+			}
+			dpm_mysql_error (func, "mysql_store_res", dbfd);
+			return (-1);
+		}
+		break;
 	}
 	return (0);
 }
@@ -1382,8 +1534,7 @@ DBLISTPTR *dblistptr;
 		 STATUS, ERRSTRING, GROUPS \
 		FROM dpm_req \
 		WHERE ctime = %d \
-		AND (status = %d OR status >= %d) \
-		FOR UPDATE";
+		AND (status = %d OR status >= %d)";
 	static char query4updlim1[] =
 		"SELECT ROWID, \
 		 R_ORDINAL, R_TOKEN, R_UID, \
@@ -1395,8 +1546,7 @@ DBLISTPTR *dblistptr;
 		FROM dpm_req \
 		WHERE ctime <= %d \
 		AND (status = %d OR status >= %d) \
-		LIMIT %d \
-		FOR UPDATE";
+		LIMIT %d";
 	static char query4updlim2[] =
 		"SELECT ROWID, \
 		 R_ORDINAL, R_TOKEN, R_UID, \
@@ -1408,8 +1558,7 @@ DBLISTPTR *dblistptr;
 		FROM dpm_req \
 		WHERE ctime BETWEEN %d AND %d \
 		AND (status = %d OR status >= %d) \
-		LIMIT %d \
-		FOR UPDATE";
+		LIMIT %d";
 	MYSQL_ROW row;
 	char sql_stmt[1024];
 
@@ -1659,7 +1808,7 @@ struct dpm_copy_filereq *cpr_entry;
 	    u64tostr (cpr_entry->actual_size, filesize_str, -1),
 	    cpr_entry->status, escaped_msg);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -1696,7 +1845,7 @@ struct dpm_fs *fs_entry;
 	    escaped_poolname, escaped_server, escaped_fs,
 	    fs_entry->status, fs_entry->weight);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -1759,7 +1908,7 @@ struct dpm_get_filereq *gfr_entry;
 	    u64tostr (gfr_entry->actual_size, filesize_str, -1),
 	    gfr_entry->status, escaped_msg);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -1817,7 +1966,7 @@ struct dpm_req *dpm_req;
 	    dpm_req->ctime, dpm_req->stime, dpm_req->etime,
 	    dpm_req->status, escaped_msg, escaped_groups);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -1887,7 +2036,7 @@ struct dpm_put_filereq *pfr_entry;
 	    u64tostr (pfr_entry->actual_size, filesize_str, -1),
 	    pfr_entry->status, escaped_msg);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -1957,7 +2106,7 @@ struct dpm_pool *pool_entry;
 	    escaped_groups,
 	    escaped_retpolicy, escaped_stype);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -2020,7 +2169,7 @@ struct dpm_space_reserv *dpm_spcmd;
 	    escaped_poolname, dpm_spcmd->assign_time, dpm_spcmd->expire_time,
 	    escaped_groups);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -2078,7 +2227,7 @@ struct dpm_req *dpm_req;
 	    dpm_req->ctime, dpm_req->stime, dpm_req->etime,
 	    dpm_req->status, escaped_msg, escaped_groups);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		if (mysql_errno (&dbfd->mysql) == ER_DUP_ENTRY)
 			serrno = EEXIST;
 		else {
@@ -2553,7 +2702,7 @@ dpm_start_tr(s, dbfd)
 int s;
 struct dpm_dbfd *dbfd;
 {
-	(void) mysql_query (&dbfd->mysql, "BEGIN");
+	(void) dm_mysql_query (&dbfd->mysql, "BEGIN");
 	dbfd->tr_started = 1;
 	return (0);
 }
@@ -2562,41 +2711,39 @@ dpm_unique_id(dbfd, unique_id)
 struct dpm_dbfd *dbfd;
 u_signed64 *unique_id;
 {
+	my_ulonglong arows;
 	char func[16];
 	static char insert_stmt[] =
-		"INSERT INTO dpm_unique_id (ID) VALUES (1)";
-	static char query_stmt[] =
-		"SELECT ID FROM dpm_unique_id FOR UPDATE";
-	MYSQL_RES *res;
-	MYSQL_ROW row;
-	char sql_stmt[1024];
-	u_signed64 uniqueid;
-	char uniqueid_str[21];
+		"INSERT INTO dpm_unique_id (ID) SELECT 0 FROM DUAL WHERE NOT \
+		 EXISTS (SELECT * FROM dpm_unique_id)";
 	static char update_stmt[] =
-		"UPDATE dpm_unique_id SET ID = %s";
+		"UPDATE dpm_unique_id SET ID = LAST_INSERT_ID(ID + 1)";
 
 	strcpy (func, "dpm_unique_id");
 
-	if (dpm_exec_query (func, dbfd, query_stmt, &res))
+	if (dm_mysql_query (&dbfd->mysql, update_stmt)) {
+		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
-	if ((row = mysql_fetch_row (res)) == NULL) {
-		mysql_free_result (res);
-		if (mysql_query (&dbfd->mysql, insert_stmt)) {
+	}
+
+	arows = mysql_affected_rows (&dbfd->mysql);
+	if (!arows) {
+		if (dm_mysql_query (&dbfd->mysql, insert_stmt)) {
 			dpm_mysql_error (func, "INSERT", dbfd);
 			return (-1);
 		}
-		*unique_id = 1;
-	} else {
-		uniqueid = strtou64 (row[0]) + 1;
-		mysql_free_result (res);
-		dm_snprintf (sql_stmt, sizeof(sql_stmt), update_stmt,
-		    u64tostr (uniqueid, uniqueid_str, -1));
-		if (mysql_query (&dbfd->mysql, sql_stmt)) {
+		if (dm_mysql_query (&dbfd->mysql, update_stmt)) {
 			dpm_mysql_error (func, "UPDATE", dbfd);
 			return (-1);
 		}
-		*unique_id = uniqueid;
+		arows = mysql_affected_rows (&dbfd->mysql);
+		if (!arows) {
+			serrno = SEINTERNAL;
+			return (-1);
+		}
 	}
+
+	*unique_id = mysql_insert_id (&dbfd->mysql);
 	return (0);
 }
 
@@ -2629,7 +2776,7 @@ struct dpm_copy_filereq *cpr_entry;
 	    u64tostr (cpr_entry->actual_size, filesize_str, -1),
 	    cpr_entry->status, escaped_msg, *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
@@ -2652,7 +2799,7 @@ struct dpm_fs *fs_entry;
 	dm_snprintf (sql_stmt, sizeof(sql_stmt), update_stmt,
 	    fs_entry->status, fs_entry->weight, *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
@@ -2696,7 +2843,7 @@ struct dpm_get_filereq *gfr_entry;
 	    u64tostr (gfr_entry->actual_size, filesize_str, -1),
 	    gfr_entry->status, escaped_msg, *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
@@ -2725,7 +2872,7 @@ struct dpm_req *dpm_req;
 	    dpm_req->status, escaped_msg,
 	    *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
@@ -2771,7 +2918,7 @@ struct dpm_put_filereq *pfr_entry;
 	    u64tostr (pfr_entry->actual_size, filesize_str, -1),
 	    pfr_entry->status, escaped_msg, *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
@@ -2833,7 +2980,7 @@ struct dpm_pool *pool_entry;
 	    escaped_groups,
 	    escaped_retpolicy, escaped_stype, *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
@@ -2872,7 +3019,7 @@ struct dpm_space_reserv *dpm_spcmd;
 	    dpm_spcmd->assign_time, dpm_spcmd->expire_time,
 	    escaped_groups, *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
@@ -2901,7 +3048,7 @@ struct dpm_req *dpm_req;
 	    dpm_req->status, escaped_msg,
 	    *rec_addr);
 
-	if (mysql_query (&dbfd->mysql, sql_stmt)) {
+	if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 		dpm_mysql_error (func, "UPDATE", dbfd);
 		return (-1);
 	}
