@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_proclreq.c,v $ $Revision: 4682 $ $Date: 2011-07-27 16:21:37 +0200 (Wed, 27 Jul 2011) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_proclreq.c,v $ $Revision: 8692 $ $Date: 2013-04-22 11:33:34 +0200 (Mon, 22 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -804,7 +804,6 @@ int nbgids;
 gid_t *gids;
 {
 	int c;
-	int cns_in_trans;
 	time_t current_time;
 	int defpintime;
 	struct dpm_space_reserv dpm_spcmd;
@@ -813,7 +812,7 @@ gid_t *gids;
 	char **excl_fs = NULL;
 	char **excl_host = NULL;
 	int excl_mutex_taken = 0;
-	int file_created;
+	int file_create;
 	struct Cns_fileid file_uniqueid;
 	char fs[80];
 	char func[17];
@@ -836,7 +835,6 @@ gid_t *gids;
 	dpm_dbrec_addr rec_addrf;
 	dpm_dbrec_addr rec_addrs;
 	struct Cns_filereplicax *rep_entries = NULL;
-	int replica_added;
 	u_signed64 reqsize;
 	gid_t sav_rgid = -1;
 	gid_t *s_gids;
@@ -903,12 +901,10 @@ gid_t *gids;
 		(void) dpm_update_pfr_entry (&thip->dbfd, &rec_addrf, &pfr_entry);
 		(void) dpm_end_tr (&thip->dbfd);
 
-		cns_in_trans = 0;
-		file_created = 0;
+		file_create = 0;
 		nbreplicas = 0;
 		pfn[0] = '\0';
 		poolname[0] = '\0';
-		replica_added = 0;
 		rep_entries = NULL;
 		u_space_updated = 0;
 		if (strncmp (pfr_entry.to_surl, "srm://", 6) == 0) {
@@ -991,72 +987,37 @@ gid_t *gids;
 					goto upd_pfr;
 				}
 				for (j = 0; j < nbreplicas; j++) {
-					file_uniqueid.fileid = (rep_entries + i)->fileid;
+					file_uniqueid.fileid = (rep_entries + j)->fileid;
 					reqsize = 0;
-					if ((rep_entries + i)->status == 'P') {
+					if ((rep_entries + j)->status == 'P') {
 						(void) dpm_start_tr (thip->s, &thip->dbfd);
 						if (dpm_get_pfr_by_pfn (&thip->dbfd,
-						    (rep_entries + i)->sfn, &old_pfr_entry,
+						    (rep_entries + j)->sfn, &old_pfr_entry,
 						    1, &rec_addr) == 0) {
 							old_status = old_pfr_entry.status;
 							old_pfr_entry.status = DPM_ABORTED;
+							snprintf (old_pfr_entry.errstring,
+							    sizeof(old_pfr_entry.errstring),
+							    "Overwritten by request %s", dpm_req->r_token);
 							if (nbreplicas == 1 &&
 							    old_pfr_entry.actual_size)
-								file_created = 1;
+								file_create = 1;
 							dpm_update_pfr_entry (&thip->dbfd,
 							    &rec_addr, &old_pfr_entry);
-							(void) dpm_end_tr (&thip->dbfd);
 							if (old_status == DPM_READY || old_status == DPM_RUNNING)
 								reqsize = old_pfr_entry.requested_size;
 						}
+						(void) dpm_end_tr (&thip->dbfd);
 					}
 					dpm_rm_onereplica (&file_uniqueid,
-					    (rep_entries + i)->sfn, reqsize, &status,
-					    &thip->dbfd, (rep_entries + i)->setname, 0, -1, NULL);
+					    (rep_entries + j)->sfn, reqsize, &status,
+					    &thip->dbfd, (rep_entries + j)->setname, 0, -1, NULL);
 				}
 				free (rep_entries);
 			}
 
-			/* Start name server transaction */
-
-			if (Cns_starttrans (NULL, NULL) < 0) {
-				pfr_entry.status = DPM_FAILED | serrno;
-				goto upd_pfr;
-			}
-			cns_in_trans = 1;
-
-			/* Create an entry in the DPM file catalog master table */
-
-			dpmlogit (func, "calling Cns_creatx\n");
-			if (Cns_creatx (sfn, 0664, &file_uniqueid) < 0) {
-				if (serrno != ENOENT) {
-					pfr_entry.status = DPM_FAILED | serrno;
-					goto upd_pfr;
-				}
-				(void) Cns_aborttrans ();
-				cns_in_trans = 0;
-				sprintf (tmpbuf, "%d", dpm_req->r_ordinal);
-				if (strcmp (dpm_req->r_token, tmpbuf)) {
-					pfr_entry.status = DPM_FAILED | ENOENT;
-					goto upd_pfr;
-				}
-				/* SRM v1 request, try to create the missing directories on the fly */
-				if (dpm_makedirp (sfn, dpm_req->r_uid, dpm_req->r_gid) < 0) {
-					pfr_entry.status = DPM_FAILED | serrno;
-					goto upd_pfr;
-				}
-				if (Cns_starttrans (NULL, NULL) < 0) {
-					pfr_entry.status = DPM_FAILED | serrno;
-					goto upd_pfr;
-				}
-				cns_in_trans = 1;
-				if (Cns_creatx (sfn, 0664, &file_uniqueid) < 0) {
-					pfr_entry.status = DPM_FAILED | serrno;
-					goto upd_pfr;
-				}
-			}
 			if (c < 0)			/* file did not exist */
-				file_created = 1;	/* but has now been successfully created */
+				file_create = 1;	/* but will be created */
 		}
 
 		/* Allocate space */
@@ -1106,8 +1067,10 @@ gid_t *gids;
 			pfr_entry.status = DPM_FAILED | ENOSPC;
 			goto upd_pfr;
 		}
-		if (excl_mutex_taken)
+		if (excl_mutex_taken) {
 			(void) Cthread_mutex_unlock (&excl_mutex);
+			excl_mutex_taken = 0;
+		}
 
 		if (*pfr_entry.s_token) {
 			if (dpm_spcmd.u_space < 0 ||
@@ -1209,15 +1172,44 @@ gid_t *gids;
 		    strrchr (pfr_entry.to_surl, '/') + 1,
 		    dpm_req->r_ordinal, pfr_entry.f_ordinal);
 
-		/* Add replica entry */
+		if (dpm_req->flags != 4) {
+			if (Cthread_mutex_lock (&excl_mutex) < 0) {
+				pfr_entry.status = DPM_FAILED | serrno;
+				goto upd_pfr;
+			}
+			excl_mutex_taken = 1;
 
-		if (Cns_addreplicax (NULL, &file_uniqueid, server, pfn, 'P',
-		    pfr_entry.f_type, poolname, fs,
-		    (dpm_req->flags == 4) ? 'S' : 'P', pfr_entry.s_token) < 0) {
-			pfr_entry.status = DPM_FAILED | serrno;
-			goto upd_pfr;
+			/* Create an entry in the DPM file catalog master table */
+
+			dpmlogit (func, "calling Cns_creatx\n");
+			if (Cns_creatx (sfn, 0664, &file_uniqueid) < 0) {
+				if (serrno != ENOENT) {
+					if (serrno == EEXIST && dpm_req->flags) {	/* overwrite option set */
+						pfr_entry.status = DPM_ABORTED;
+						snprintf (pfr_entry.errstring,
+						    sizeof(pfr_entry.errstring),
+						    "Overwritten by another request");
+					} else {
+						pfr_entry.status = DPM_FAILED | serrno;
+					}
+					goto upd_pfr;
+				}
+				sprintf (tmpbuf, "%d", dpm_req->r_ordinal);
+				if (strcmp (dpm_req->r_token, tmpbuf)) {
+					pfr_entry.status = DPM_FAILED | ENOENT;
+					goto upd_pfr;
+				}
+				/* SRM v1 request, try to create the missing directories on the fly */
+				if (dpm_makedirp (sfn, dpm_req->r_uid, dpm_req->r_gid) < 0) {
+					pfr_entry.status = DPM_FAILED | serrno;
+					goto upd_pfr;
+				}
+				if (Cns_creatx (sfn, 0664, &file_uniqueid) < 0) {
+					pfr_entry.status = DPM_FAILED | serrno;
+					goto upd_pfr;
+				}
+			}
 		}
-		replica_added = 1;
 
 		/* Update server/pfn/status in pfr_entry */
 
@@ -1237,17 +1229,10 @@ gid_t *gids;
 			pfr_entry.lifetime = maxpintime;
 		strcpy (pfr_entry.server, server);
 		strcpy (pfr_entry.pfn, pfn);
-		if (file_created)
+		if (file_create)
 			pfr_entry.actual_size = LLONG_MAX;
 		pfr_entry.status = DPM_READY;
 upd_pfr:
-		if (dpm_req->flags == 4) {	/* for replication */
-			if (excl_mutex_taken)
-				(void) Cthread_mutex_unlock (&excl_mutex);
-			free (excl_fs);
-			free (excl_host);
-			free (rep_entries);
-		}
 		(void) dpm_start_tr (thip->s, &thip->dbfd);
 		if (dpm_get_pfr_by_fullid (&thip->dbfd, dpm_req->r_token, i,
 		    &old_pfr_entry, 1, &rec_addrf) < 0) {
@@ -1257,9 +1242,15 @@ upd_pfr:
 			if (old_pfr_entry.status == DPM_ACTIVE) {
 				(void) dpm_update_pfr_entry (&thip->dbfd, &rec_addrf,
 				    &pfr_entry);
-				if (pfr_entry.status == DPM_READY && cns_in_trans) {
-					(void) Cns_endtrans ();
-					cns_in_trans = 0;
+				if (pfr_entry.status == DPM_READY) {
+					/* Add replica entry */
+					if (Cns_addreplicax (NULL, &file_uniqueid, server, pfn, 'P',
+					    pfr_entry.f_type, poolname, fs,
+					    (dpm_req->flags == 4) ? 'S' : 'P', pfr_entry.s_token) < 0) {
+						pfr_entry.status = DPM_FAILED | serrno;
+						dpm_abort_tr (&thip->dbfd);
+						goto upd_pfr;
+					}
 				}
 			} else {
 				if ((pfr_entry.status & 0xF000) != DPM_FAILED)
@@ -1267,20 +1258,25 @@ upd_pfr:
 			}
 			(void) dpm_end_tr (&thip->dbfd);
 		}
-		if ((pfr_entry.status & 0xF000) == DPM_FAILED) {
+		if (pfr_entry.status != DPM_READY) {
 		       	if (*pfn) {
 				if (u_space_updated)
 					(void) dpm_upd_u_space (&thip->dbfd, pfr_entry.s_token,
 					    pfr_entry.requested_size);
 				dpm_updfreespace (pfn, pfr_entry.requested_size, pfr_entry.s_token, 0);
 			}
-			if (replica_added)
-				(void) Cns_delreplica (NULL, &file_uniqueid, pfn);
-			if (file_created)
+			if (file_create)
 				(void) Cns_unlink (sfn);
 		}
-		if (cns_in_trans)
-			(void) Cns_endtrans ();
+		if (excl_mutex_taken) {
+			(void) Cthread_mutex_unlock (&excl_mutex);
+			excl_mutex_taken = 0;
+		}
+		if (dpm_req->flags == 4) {	/* for replication */
+			free (excl_fs);
+			free (excl_host);
+			free (rep_entries);
+		}
 		if (pfr_entry.status != DPM_READY) {
 			dpmlogit (func, "%s: %s\n", pfr_entry.to_surl,
 			    *pfr_entry.errstring ? pfr_entry.errstring :
