@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: poolmgr.c,v $ $Revision: 6384 $ $Date: 2012-06-06 16:39:34 +0200 (Wed, 06 Jun 2012) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: poolmgr.c,v $ $Revision: 8679 $ $Date: 2013-04-19 09:35:01 +0200 (Fri, 19 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -964,7 +964,7 @@ dpm_selectfs(gid_t gid, char f_type, char ret_policy, char ac_latency, char **ex
 	return (0);
 }
 
-dpm_selectpool(gid_t gid, char f_type, char ret_policy, char ac_latency, u_signed64 reqsize, char *poolname, char *errstring)
+dpm_selectpool(gid_t gid, char f_type, char ret_policy, char ac_latency, u_signed64 reqsize, char *poolname, char *errstring, int ocpool)
 {
 	int i;
 	struct dpm_pool *poolp;
@@ -1004,11 +1004,13 @@ dpm_selectpool(gid_t gid, char f_type, char ret_policy, char ac_latency, u_signe
 			serrno = EACCES;
 			return (-1);
 		}
-		if ((pools + i)->capacity <= 0 || (pools + i)->free < 0 ||
-		    (pools + i)->free < reqsize) {
-			(void) Cthread_mutex_unlock (&poolconf_mutex);
-			serrno = ENOSPC;
-			return (-1);
+		if (!ocpool) {
+			if ((pools + i)->capacity <= 0 || (pools + i)->free < 0 ||
+			    (pools + i)->free < reqsize) {
+				(void) Cthread_mutex_unlock (&poolconf_mutex);
+				serrno = ENOSPC;
+				return (-1);
+			}
 		}
 	}
 	poolp = pools + i;
