@@ -1,5 +1,5 @@
 /*
- * $Id: dpm_server.h 4555 2011-06-29 11:54:25Z baud $
+ * $Id: dpm_server.h 8679 2013-04-19 07:35:01Z dhsmith $
  */
 
 /*
@@ -8,7 +8,7 @@
  */
 
 /*
- * @(#)$RCSfile: dpm_server.h,v $ $Revision: 4555 $ $Date: 2011-06-29 13:54:25 +0200 (Wed, 29 Jun 2011) $ CERN IT-GD/CT Jean-Philippe Baud
+ * @(#)$RCSfile: dpm_server.h,v $ $Revision: 8679 $ $Date: 2013-04-19 09:35:01 +0200 (Fri, 19 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud
  */
 
 #ifndef _DPM_SERVER_H
@@ -291,7 +291,7 @@ EXTERN_C int dpm_reset_gc_entry _PROTO((int));
 EXTERN_C int dpm_rmfsfrompoolconf _PROTO((char *, char *));
 EXTERN_C int dpm_rmpoolfrompoolconf _PROTO((char *));
 EXTERN_C int dpm_selectfs _PROTO((gid_t, char, char, char, char **, char **, u_signed64 *, char *, char *, char *, int *));
-EXTERN_C int dpm_selectpool _PROTO((gid_t, char, char, char, u_signed64, char *, char *));
+EXTERN_C int dpm_selectpool _PROTO((gid_t, char, char, char, u_signed64, char *, char *, int));
 EXTERN_C int dpm_start_tr _PROTO((int, struct dpm_dbfd *));
 EXTERN_C int dpm_stop_gc _PROTO((char *));
 EXTERN_C int dpm_unique_id _PROTO((struct dpm_dbfd *, u_signed64 *));
