@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: Cns_main.c,v $ $Revision: 6490 $ $Date: 2012-06-25 16:20:06 +0200 (Mon, 25 Jun 2012) $ CERN IT-PDP/DM Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: Cns_main.c,v $ $Revision: 8852 $ $Date: 2013-05-28 11:33:35 +0200 (Tue, 28 May 2013) $ CERN IT-PDP/DM Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -42,11 +42,12 @@ static char sccsid[] = "@(#)$RCSfile: Cns_main.c,v $ $Revision: 6490 $ $Date: 20
 #include "MsgProducerCAPI.h"
 #endif
 
+#include "LcgdmDbPool_C.h"
 
 /*
  * Maximum number of sockets on which the server can listen
  */
-#define MAX_LISTEN_SOCKS        16
+#define MAX_LISTEN_SOCKS	16
 
 int being_shutdown;
 char *cmd;
@@ -62,6 +63,7 @@ struct Cns_srv_thread_info *Cns_srv_thread_info;
 extern int Cdomainname (char *, int);
 extern int isTrustedHost (int, char *, char *, char *, char *);
 static int na_key = -1;
+struct Cns_srv_thread_info *Cns_srv_dbfd_info;
 
 Cns_main(main_args)
 struct main_args *main_args;
@@ -70,6 +72,7 @@ struct main_args *main_args;
 	struct Cns_dbfd dbfd;
 	struct Cns_file_metadata direntry;
 	void *doit(void *);
+	void *doit_dbpool(void *);
 	char *dp;
 	struct sockaddr_storage from;
 	int fromlen;
@@ -77,6 +80,8 @@ struct main_args *main_args;
 	int i;
 	int ipool;
 	int nbthreads = CNS_NBTHREADS;
+	int nbthreadsmult = CNS_NBTHREADSMULT;
+
 	int on = 1;	/* for REUSEADDR and IPV6_V6ONLY */
 	char *nsconfigfile = NULL;
 	char *p;
@@ -158,7 +163,23 @@ struct main_args *main_args;
 		strcat (localhost, localdomain);
 	}
 
+	if (getenv("NBTHREADSMULT")) {
+		nbthreadsmult = strtol(getenv("NBTHREADSMULT"), NULL, 10);
 
+		if (nbthreadsmult <= 1) { /* This disables the dbpool enhancements completely */
+			nslogit (func, "Dbpool enhancements are now off.\n");
+			nbthreadsmult = 1;
+		}
+		else {
+			if ((nbthreadsmult < 0) || (nbthreads*nbthreadsmult >= 1024) ) {
+				nslogit (func, "Invalid thread multiplier: %s\n", nbthreadsmult);
+				exit (USERR);
+			}
+		}
+	}
+
+	nslogit (func, "Eff. DB connections: %d, Multiplied thread pool: %d\n", nbthreads, nbthreads*nbthreadsmult);
+      
 
 
 #ifdef SEMSG
@@ -189,6 +210,7 @@ struct main_args *main_args;
 	dbfd.idx = nbthreads;
 	if (Cns_opendb (&dbfd) < 0)
 		return (SYERR);
+
 	if (Cns_get_fmd_by_fullid (&dbfd, (u_signed64) 0, "/", &direntry, 0, NULL) < 0) {
 		if (serrno != ENOENT)
 			return (SYERR);
@@ -213,20 +235,44 @@ struct main_args *main_args;
 
 	/* create a pool of threads */
 
-	if ((ipool = Cpool_create (nbthreads, NULL)) < 0) {
+	if ((ipool = Cpool_create (nbthreads*nbthreadsmult, NULL)) < 0) {
 		nslogit (func, NS002, "Cpool_create", sstrerror(serrno));
 		return (SYERR);
 	}
 	if ((Cns_srv_thread_info =
-	    calloc (nbthreads, sizeof(struct Cns_srv_thread_info))) == NULL) {
+	    calloc (nbthreads*nbthreadsmult, sizeof(struct Cns_srv_thread_info))) == NULL) {
 		nslogit (func, NS002, "calloc", strerror(errno));
 		return (SYERR);
 	}
-	for (i = 0; i < nbthreads; i++) {
+	for (i = 0; i < nbthreads*nbthreadsmult; i++) {
 		(Cns_srv_thread_info+i)->s = -1;
 		(Cns_srv_thread_info+i)->dbfd.idx = i;
 	}
 
+	if (nbthreadsmult > 1) { /* This is useless but easier to read */
+		if ((Cns_srv_dbfd_info =
+			calloc (nbthreads, sizeof(struct Cns_srv_thread_info))) == NULL) {
+			nslogit (func, NS002, "calloc", strerror(errno));
+			return (SYERR);
+		}
+		for (i = 0; i < nbthreads; i++) {
+			(Cns_srv_dbfd_info+i)->dbfd.idx = i;
+			if (Cns_opendb (&(Cns_srv_dbfd_info+i)->dbfd) < 0) {
+				c = serrno;
+				nslogit (func, NS002, "db open error: %d\n", c);
+				return (CONFERR);
+			}
+			(Cns_srv_dbfd_info+i)->db_open_done = 1;
+			(Cns_srv_dbfd_info+i)->last_db_use = time (0);
+			LcgdmDbPool_AddDbfd((void *)(Cns_srv_dbfd_info+i));
+		}
+	}
+
+	nslogit (func, "DB looks sane. Pools created.\n");
+
+
+
+	
 	FD_ZERO (&readmask);
 	FD_ZERO (&readfd);
 #if ! defined(_WIN32)
@@ -309,7 +355,7 @@ struct main_args *main_args;
 		}
 		listen_socks[num_listen_socks] = s;
 		++num_listen_socks;
-		listen (s, 5);
+		listen (s, 128);
 	}
 	freeaddrinfo (aitop);
 
@@ -331,20 +377,54 @@ struct main_args *main_args;
 
 	while (1) {
 		if (being_shutdown) {
-			int nb_active_threads = 0;
-			for (i = 0; i < nbthreads; i++) {
-				if ((Cns_srv_thread_info+i)->s >= 0) {
-					nb_active_threads++;
-					continue;
-				}
-				if ((Cns_srv_thread_info+i)->db_open_done)
-					(void) Cns_closedb (&(Cns_srv_thread_info+i)->dbfd);
+			if (nbthreadsmult > 1) {
+				int nb_active_threads = 0;
+				for (i = 0; i < nbthreads*nbthreadsmult; i++) {
+					/* Fantastic... these items are not protected since ever */
+					if ((Cns_srv_thread_info+i)->s >= 0) {
+						nb_active_threads++;
+						continue;
+					}       
+					if ((Cns_srv_thread_info+i)->db_open_done)
+						(void) Cns_closedb (&(Cns_srv_thread_info+i)->dbfd);
+				}	       
+				
+				if (nb_active_threads == 0) {
+					/* Cleanup various things, we are going to exit */
+#ifdef CSEC			     
+					/* Cleanup the Csec instances */
+					for (i = 0; i < nbthreads*nbthreadsmult; i++) {
+						(void) Csec_clearContext (&(Cns_srv_thread_info+i)->sec_ctx);
+					}       
+#endif				  
+
+					/* Close all the db instances */
+					for (i = 0; i < nbthreads; i++) {
+						(void) Cns_closedb (&(Cns_srv_dbfd_info+i)->dbfd);
+					}       
+					
+					return (0);
+				}       
+			}       
+			else {
+			
+				/* The old code, buggy and taboo */
+				int nb_active_threads = 0;
+				for (i = 0; i < nbthreads; i++) {
+					if ((Cns_srv_thread_info+i)->s >= 0) {
+						nb_active_threads++;
+						continue;
+					}       
+					if ((Cns_srv_thread_info+i)->db_open_done)
+						(void) Cns_closedb (&(Cns_srv_thread_info+i)->dbfd);
 #ifdef CSEC
-				(void) Csec_clearContext (&(Cns_srv_thread_info+i)->sec_ctx);
+					(void) Csec_clearContext (&(Cns_srv_thread_info+i)->sec_ctx);
 #endif
+				}
+				if (nb_active_threads == 0)
+					return (0);
 			}
-			if (nb_active_threads == 0)
-				return (0);
+
 		}
 		for (i=0; i < num_listen_socks; ++i) {
 			s = listen_socks[i];
@@ -362,11 +442,25 @@ struct main_args *main_args;
 						return (SYERR);
 				}
 				(Cns_srv_thread_info+thread_index)->s = rqfd;
-				if (Cpool_assign (ipool, &doit,
-			    	Cns_srv_thread_info+thread_index, 1) < 0) {
-					(Cns_srv_thread_info+thread_index)->s = -1;
-					nslogit (func, NS002, "Cpool_assign", sstrerror(serrno));
-					return (SYERR);
+
+				if (nbthreadsmult > 1) {
+					/* Use the dbpool */
+					if (Cpool_assign (ipool, &doit_dbpool,
+					    Cns_srv_thread_info+thread_index, 1) < 0) {
+						(Cns_srv_thread_info+thread_index)->s = -1;
+						nslogit (func, NS002, "Cpool_assign", sstrerror(serrno));
+						return (SYERR);
+					}
+				}
+				else {
+					/* Use the good old, untouched code */
+					if (Cpool_assign (ipool, &doit,
+					Cns_srv_thread_info+thread_index, 1) < 0) {
+						(Cns_srv_thread_info+thread_index)->s = -1;
+						nslogit (func, NS002, "Cpool_assign", sstrerror(serrno));
+						return (SYERR);
+					}
+
 				}
 			}
 		}
@@ -377,6 +471,8 @@ struct main_args *main_args;
 			FD_ZERO (&readfd);
 		}
 	}
+
+
 }
 
 main(argc, argv)
@@ -397,6 +493,159 @@ char **argv;
 #endif
 }
 
+
+
+
+void *
+doit_dbpool(arg)
+void *arg;
+{
+	int alloced_gids = 0;
+	int c;
+	const char *clienthost = NULL;
+	const char *clientip;
+	const char *clientname;
+	int magic;
+	char *req_data;
+	char reqbuf[REQBUFSZ*2];
+	int req_type = 0;
+	struct Cns_srv_thread_info *thip = (struct Cns_srv_thread_info *) arg;
+	struct Cns_srv_thread_info *dbfdok = NULL;
+	struct Cns_srv_thread_info dbsave,dbsave2,thipsave;
+
+
+	(void) Cgetnetaddress (thip->s, NULL, 0, &na_key, &clientip, &clientname, 0, 0);
+	if (clientip == NULL)
+		clientip = "unknown";
+	if (clientname == NULL)
+		clientname = "unknown";
+
+#ifdef CSEC
+	Csec_server_reinitContext (&thip->sec_ctx, CSEC_SERVICE_TYPE_HOST, NULL);
+	if (Csec_server_establishContext (&thip->sec_ctx, thip->s) < 0) {
+		nslogit (func, "[%s] (%s): Could not establish an authenticated connection: %s !\n",
+			clientip, clientname, Csec_getErrorMessageSummary (LOGBUFSZ-140));
+		sendrep (thip->s, CNS_RC, ESEC_NO_CONTEXT);
+		thip->s = -1;
+		return NULL;
+	}
+	thip->Csec_gids = NULL;
+	Csec_server_getClientId (&thip->sec_ctx, &thip->Csec_mech, &thip->Csec_auth_id);
+	if (strcmp (thip->Csec_mech, "ID") == 0 ||
+	  Csec_isIdAService (thip->Csec_mech, thip->Csec_auth_id) >= 0) {
+		if (isTrustedHost (thip->s, localhost, localdomain, CNS_SCE, "TRUST")) {
+			if (Csec_server_getAuthorizationId (&thip->sec_ctx,
+			    &thip->Csec_mech, &thip->Csec_auth_id) < 0) {
+				thip->Csec_uid = 0;
+				thip->Csec_gid = 0;
+				thip->Csec_nbgids = 1;
+				thip->Csec_gids = &thip->Csec_gid;
+#ifndef VIRTUAL_ID
+			} else if (Csec_mapToLocalUser (thip->Csec_mech, thip->Csec_auth_id,
+			    NULL, 0, &thip->Csec_uid, &thip->Csec_gid) < 0) {
+				nslogit (func, "[%s] (%s): Could not map (%s,\"%s\") to local user: %s !\n",
+				    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id, sstrerror (serrno));
+				sendrep (thip->s, CNS_RC, serrno);
+				thip->s = -1;
+				return NULL;
+#else
+			} else {	/* mapping will be done later */
+			       thip->Csec_uid = (uid_t) -1;
+				thip->Csec_gid = (gid_t) -1;
+				alloced_gids = 1;
+#endif
+			}
+		} else {
+			nslogit (func, "[%s] (%s): Host is not trusted, identity provided was (%s,\"%s\")\n",
+			    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id);
+			sendrep (thip->s, CNS_RC, EACCES);
+			thip->s = -1;
+			return NULL;
+		}
+#ifndef VIRTUAL_ID
+	} else if (Csec_mapToLocalUser (thip->Csec_mech, thip->Csec_auth_id,
+	    NULL, 0, &thip->Csec_uid, &thip->Csec_gid) < 0) {
+		nslogit (func, "[%s] (%s): Could not map (%s,\"%s\") to local user: %s !\n",
+		    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id, sstrerror (serrno));
+		sendrep (thip->s, CNS_RC, serrno);
+		thip->s = -1;
+		return NULL;
+#else
+	} else {	/* mapping will be done later */
+		thip->Csec_uid = (uid_t) -1;
+		thip->Csec_gid = (gid_t) -1;
+		alloced_gids = 1;
+#endif
+	}
+#endif
+	req_data = reqbuf;
+
+
+
+	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &clienthost)) == 0) {
+
+		/* Get a thip containing a good db connection
+		   Only the dbfd field will be useful */
+		dbfdok = (struct Cns_srv_thread_info *)LcgdmDbPool_GetDbfd();
+		if (!dbfdok) {
+			sendrep (thip->s, MSG_ERR, "no more db instances: %d\n", c);
+			netclose (thip->s);
+			thip->s = -1;
+			return (NULL);
+		}
+
+		/* Save the db conn pool and the per thread structures */
+		dbsave = *dbfdok;
+		thipsave = *thip;
+
+		/* Reset dbfdok keeping only db elements from db pool entry */
+		*dbfdok = *thip;
+		dbfdok->dbfd = dbsave.dbfd;
+		dbfdok->db_open_done = dbsave.db_open_done;
+		dbfdok->last_db_use = dbsave.last_db_use;
+
+		c = procreq (magic, req_type, req_data, clienthost, dbfdok);
+
+		dbsave2 = *dbfdok;
+		*dbfdok = dbsave;
+		dbfdok->dbfd = dbsave2.dbfd;
+		dbfdok->db_open_done = dbsave2.db_open_done;
+		dbfdok->last_db_use = dbsave2.last_db_use;
+		LcgdmDbPool_AddDbfd((void *) dbfdok);
+
+		*thip = dbsave2;
+		thip->dbfd = thipsave.dbfd;
+		thip->db_open_done = thipsave.db_open_done;
+		thip->last_db_use = thipsave.last_db_use;
+		
+		sendrep (thip->s, CNS_RC, c);
+
+	} else {
+		nslogit (func, "[%s] (%s): Failure getting the request: %s\n",
+			clientip, clientname, sstrerror(c));
+		if (c > 0)
+			sendrep (thip->s, CNS_RC, c);
+		else
+			netclose (thip->s);
+	}
+	if (req_data != reqbuf)
+		free (req_data);
+	if (alloced_gids)
+		free (thip->Csec_gids);
+	thip->s = -1;
+	return (NULL);
+}
+
+
+
+
+
+
+
+
+
+
+
 void *
 doit(arg)
 void *arg;
