@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: send2nsd.c,v $ $Revision: 6571 $ $Date: 2012-07-03 17:12:11 +0200 (Tue, 03 Jul 2012) $ CERN IT-PDP/DM Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: send2nsd.c,v $ $Revision: 8204 $ $Date: 2013-01-30 20:01:21 +0100 (Wed, 30 Jan 2013) $ CERN IT-PDP/DM Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -257,12 +257,8 @@ int *nbstruct;
 					Cns_errmsg (func, NS002, "send", "Operation timed out");
 					break;
 				default:
-					{
-					char *err_buffer = Csec_getErrorMessageSummary (PRTBUFSZ-48);
-					Cns_errmsg (func, NS002, "send", err_buffer);
-					free(err_buffer);
+					Cns_errmsg (func, NS002, "send", Csec_getErrorMessageSummary (PRTBUFSZ-48));
 					abortconnect = 1;
-					}
 					break;
 			}
 
