@@ -15,7 +15,7 @@
 # .. to get the help screen
 #
 
-__version__ = "0.1"
+__version__ = "1.0"
 __author__  = "Fabrizio Furano"
 
 
@@ -52,7 +52,11 @@ def addrecord(xmlroot, hostname, group, user, site, filecount, resourcecapacityu
   # update XML 
   rec = etree.SubElement(xmlroot, SR+'StorageUsageRecord')
   rid = etree.SubElement(rec, SR+'RecordIdentity')
-  rid.set(SR+"createTime", datetime.datetime.now().isoformat())
+  rid.set(SR+"createTime", datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ"))
+
+  if hostname:
+    ssys = etree.SubElement(rec, SR+"StorageSystem")
+    ssys.text = hostname
 
   recid = recordid  
   if not recid:
@@ -80,24 +84,41 @@ def addrecord(xmlroot, hostname, group, user, site, filecount, resourcecapacityu
 
   if validduration:
     e = etree.SubElement(rec, SR+"EndTime")
-    d = datetime.datetime.now() + datetime.timedelta(seconds=validduration)
-    e.text = d.isoformat()
+    d = datetime.datetime.utcnow() + datetime.timedelta(seconds=validduration)
+    e.text = d.strftime("%Y-%m-%dT%H:%M:%SZ")
 
   e = etree.SubElement(rec, SR+"StartTime")
-  e.text = datetime.datetime.now().isoformat()
+  e.text = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%SZ")
 
 
   if filecount:
     e = etree.SubElement(rec, SR+"FileCount")
     e.text = str(filecount)
 
-  if resourcecapacityused:
-   e = etree.SubElement(rec, SR+"ResourceCapacityUsed")
-   e.text = str(resourcecapacityused)
 
-  if logicalcapacityused:
-   e = etree.SubElement(rec, SR+"LogicalCapacityUsed")
-   e.text = str(logicalcapacityused)
+
+
+
+
+
+
+
+
+
+  if not resourcecapacityused:
+    resourcecapacityused = 0
+
+  e1 = etree.SubElement(rec, SR+"ResourceCapacityUsed")
+  e1.text = str(resourcecapacityused)
+
+  e3 = etree.SubElement(rec, SR+"ResourceCapacityAllocated")
+  e3.text = str(resourcecapacityused)
+
+  if not logicalcapacityused:
+    logicalcapacityused = 0
+
+  e2 = etree.SubElement(rec, SR+"LogicalCapacityUsed")
+  e2.text = str(logicalcapacityused)
 
 
 
@@ -145,18 +166,37 @@ def getreservedspacebyuid(dbconn, uid):
 
 
 
+def getdnsnames(name): 
+  d = socket.gethostbyaddr(name) 
+  names = [ d[0] ] + d[1] + d[2] 
+  return names 
 
+def resolve(name): 
+  names = getdnsnames(name) 
+  for dnsname in names: 
+    if '.' in dnsname: 
+      fullname = dnsname 
+      break 
+    else: 
+      fullname = name 
+  return fullname 
 
+def gethostname(): 
+  fullname = socket.gethostname() 
+  if '.' not in fullname: 
+    fullname = resolve(fullname) 
+  return fullname
 
 #############
 # Main code #
 #############
 
+
 parser = OptionParser()
 parser.add_option('--reportgroups', dest='reportgroups', action='store_true', default=False, help="Report about all groups")
 parser.add_option('--reportusers', dest='reportusers', action='store_true', default=False, help="Report about all users")
 parser.add_option('-v', '--debug', dest='verbose', action='count', help='Increase verbosity level for debugging (on stderr)')
-parser.add_option('--hostname', dest='hostname', default=platform.node(), help="The hostname string to use in the record. Default: this host.")
+parser.add_option('--hostname', dest='hostname', default=gethostname(), help="The hostname string to use in the record. Default: this host.")
 parser.add_option('--site', dest='site', default="", help="The site string to use in the record. Default: none.")
 parser.add_option('--recordid', dest='recordid', default=None, help="The recordid string to use in the record. Default: a newly computed unique string.")
 parser.add_option('--nsconfig', dest='nsconfig', default=None, help="Path to the NSCONFIG file where to take the db login info")
