@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: Csec_protocol_policy.c,v $ $Revision: 3714 $ $Date: 2010-08-04 08:52:54 +0200 (Wed, 04 Aug 2010) $ IT/ADC/CA Benjamin Couturier";
+static char sccsid[] = "@(#)$RCSfile: Csec_protocol_policy.c,v $ $Revision: 6384 $ $Date: 2012-06-06 16:39:34 +0200 (Wed, 06 Jun 2012) $ IT/ADC/CA Benjamin Couturier";
 #endif
 		
 /*
@@ -27,6 +27,7 @@ static char sccsid[] = "@(#)$RCSfile: Csec_protocol_policy.c,v $ $Revision: 3714
 #include <osdep.h>
 #include "Csec.h"
 #include "Csec_protocol_policy.h"
+#include "Cthread_env.h"
 
 
 EXTERN_C char DLL_DECL *getconfent _PROTO((char *, char *, int));
@@ -148,9 +149,9 @@ int Csec_client_lookup_protocols(Csec_protocol **protocols, int *nbprotocols) {
 
   /* Getting the protocol list from environment variable, configuration file
      or default value */
-  if (!((p = (char *)getenv (CSEC_MECH)) 
-	|| (p = (char *)getconfent (CSEC_CONF_SECTION, 
-				    CSEC_CONF_ENTRY_MECH, 0)))) {
+  if (!((p = Cthread_getenv (CSEC_MECH))
+	||  (p = (char *)getconfent (CSEC_CONF_SECTION,
+				      CSEC_CONF_ENTRY_MECH, 0)))) {
     p = CSEC_DEFAULT_MECHS;
     serrno = 0;
   }
@@ -229,7 +230,7 @@ int Csec_server_lookup_protocols(struct sockaddr *client_address,
 	     
   /* Getting the protocol list from environment variable, configuration file
      or default value */
-  if (!((p = (char *)getenv (CSEC_AUTH_MECH)) 
+  if (!((p = Cthread_getenv (CSEC_AUTH_MECH))
 	|| (p = (char *)getconfent (CSEC_CONF_SECTION, CSEC_CONF_ENTRY_AUTHMECH, 0)))) {
     p = CSEC_DEFAULT_MECHS;
     serrno = 0;
